/**Copyright  2012 held by
 * Robert Williams
 * Robert Poland
 * Marc Hayes
 * Coen Smits
 * Justin Becker
 * All rights reserved.
 * 
 * 
 * This file is part of Bloodmoon.

    Bloodmoon is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bloodmoon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bloodmoon.  If not, see <http://www.gnu.org/licenses/>.
 */




public abstract class Attack extends GameObject 
{
	// ==============================================
	//                  Members
	// ==============================================
	protected float m_accelX;
	protected float m_accelY;
	
	protected long m_lifeSpan;
	
	protected int m_damage;
	
	// ==============================================
	//                  Constructors
	// ==============================================

	public Attack(Vector2D initialPosition, float velocityX, float velocityY, int damage, long lifeSpan)
	{
		m_damage = damage;
		
		m_animator = new Animator(false);
				
		m_acceleration = new Vector2D(0.0f, 0.0f);
		m_velocity = new Vector2D(velocityX, velocityY);
		m_position = new Vector2D(initialPosition.GetX(), initialPosition.GetY());
		
		m_lifeSpan = lifeSpan;
		
		Initialize();
		
		m_animator.Finish();
	}
	
	public Attack(Vector2D initialPosition, int damage, long lifeSpan)
	{
		m_position = new Vector2D(initialPosition.GetX(), initialPosition.GetY());
		
		m_lifeSpan = lifeSpan;
		
		m_animator = new Animator(false);

		Initialize();		

		m_animator.Finish();
	}
	
	/**
	 * Empty constructor to allow children to implement their own constructors.  
	 * Children must initialize sprites, position, bounding box, and life span
	 */
	public Attack()
	{
		
	}
	
	// ==============================================
	//                  Overrides
	// ==============================================

	@Override
	protected void Initialize()
	{
		InitializeSprites();
		InitializeBoundingBox();
	}
	
	@Override
	public void Update(long elapsedGameTime) 
	{
		m_lifeSpan -= elapsedGameTime;
		
		m_currentImage = m_animator.CalculateNextFrameLinear(elapsedGameTime);
	}
	// ==============================================
	//                  Methods
	// ==============================================	
	
	/**
	 * Determine if the attack has finished, i.e. it's life span has reached 0
	 * @return
	 */
	public boolean IsExpired()
	{
		return m_lifeSpan < 0;
	}
	
	public int GetDamage()
	{
		return m_damage;
	}
}
