/**Copyright  2012 held by
 * Robert Williams
 * Robert Poland
 * Marc Hayes
 * Coen Smits
 * Justin Becker
 * All rights reserved.
 * 
 * 
 * This file is part of Bloodmoon.

    Bloodmoon is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bloodmoon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bloodmoon.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;


public class Bar
{
	// Members
	private int m_x = 10;
	private int m_y = 10;
	private int m_maxValue = 100;
	private int m_minValue = 0 ;
	private int m_currentValue = 100;
	private Color m_color = Color.RED;

	private BufferedImage m_barImage = null;
	private int m_width = 162;
	private int m_thickness = 16;
	private int m_xOffSet = 38; // The image has a start offset which needs to be compensated for
	private int m_yOffSet = 3; // The image has a start offset which needs to be compensated for

	// Getters
	public int getCurrentValue()
	{
		return m_currentValue;
	}
	
	public int getCurrentMax()
	{
		return m_maxValue;
	}
	
	// Setters
	public void setCurrentValue(int value)
	{
		m_currentValue = value;
	}
	
	public void setCurrentValue(int value, int maximum)
	{
		m_currentValue = value;
		m_maxValue = maximum;
	}
	
	// Constructor
	public Bar(int x, int y, int currentValue, int maxValue, Color color)
	{
		m_x = x;
		m_y = y;
		m_currentValue = currentValue;
		m_maxValue = maxValue;
		m_color = color;
		
		m_barImage = Game.LoadImage(".\\Resources\\Images\\Menu\\EmptyBar.png");
	}

	public void Paint(Graphics2D graphics)
	{
		int xWithOffset = m_x + m_xOffSet;
		int yWithOffset = m_y + m_yOffSet;
		
		graphics.setColor(m_color.darker().darker().darker());		
		graphics.fillRect(xWithOffset, yWithOffset, m_width, m_thickness);
		
		double ratio = m_currentValue / (double)Math.abs(m_maxValue - m_minValue);
		
		int widthCurrentValue = (int)(ratio * m_width);
		
		graphics.setColor(m_color);
		graphics.fillRect(xWithOffset, yWithOffset, widthCurrentValue, m_thickness);	
		
		String progress = m_currentValue + "/" + m_maxValue;
		graphics.setColor(Color.WHITE);
		graphics.drawString(progress, xWithOffset + (m_width / 2) - 20, yWithOffset + (m_thickness / 2) + 3);
		
		// Render the image on top
		graphics.drawImage(m_barImage, null, m_x, m_y);
	}
}
