/**Copyright  2012 held by
 * Robert Williams
 * Robert Poland
 * Marc Hayes
 * Coen Smits
 * Justin Becker
 * All rights reserved.
 * 
 * 
 * This file is part of Bloodmoon.

    Bloodmoon is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bloodmoon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bloodmoon.  If not, see <http://www.gnu.org/licenses/>.
 */


public class Coin extends GameObject 
{
	private final int m_value;
	protected long m_lifeSpan;
	protected boolean m_permanent;
	
	public enum CoinType
	{
		GOLD,
		SILVER,
		COPPER
	}
	
	public Coin(CoinType type, float locationX, float locationY) 
	{
		super();
		
		m_permanent = false;
		
		m_lifeSpan = 1000 * 10; // 10 secs

		if (type == CoinType.GOLD)
		{
			m_animator.LoadSpriteSheet(".\\Resources\\Images\\Treasure\\coin_gold.png");
			m_value = 3;
		}
		else if (type == CoinType.SILVER)
		{
			m_animator.LoadSpriteSheet(".\\Resources\\Images\\Treasure\\coin_silver.png");
			m_value = 2;
		}
		else if (type == CoinType.COPPER)
		{
			m_animator.LoadSpriteSheet(".\\Resources\\Images\\Treasure\\coin_copper.png");
			m_value = 1;
		}
		else
		{
			m_value = 0;
		}
		
		m_animator.AddAnimation(Action.Idle, Direction.NORTH, 0, 0, 32, 32, 8);
		
		m_position = new Vector2D(locationX, locationY);
		
		m_animator.BeginNewAction(Direction.NORTH, Action.Idle, 1000, true);
		
		m_currentImage = m_animator.CalculateNextFrameLinear(0);
		
		InitializeBoundingBox();
	}

	public int GetValue()
	{
		return m_value;
	}
	
	@Override
	public void Update(long elapsedGameTime)
	{
		m_currentImage = m_animator.CalculateNextFrameLinear(elapsedGameTime);
		
		m_lifeSpan -= elapsedGameTime;
		
		if (m_lifeSpan < 0L && !m_permanent)
		{
			SetExpired();
		}
	}

	@Override
	protected void InitializeSprites()
	{

	}

	@Override
	protected void InitializeStartingVectors() 
	{
	
	}

}
