import java.awt.AlphaComposite;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.util.LinkedList;

/**Copyright  2012 held by
 * Robert Williams
 * Robert Poland
 * Marc Hayes
 * Coen Smits
 * Justin Becker
 * All rights reserved.
 * 
 * 
 * This file is part of Bloodmoon.

    Bloodmoon is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bloodmoon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bloodmoon.  If not, see <http://www.gnu.org/licenses/>.
 */



public abstract class Creature extends GameObject 
{
	private int m_strength;
	private int m_defense;
	private int m_intelligence;

	private int m_damage;
	private int m_damageReduction;
	
	private int m_modifiedStr;
	private int m_modifiedDef;
	private int m_modifiedInt;
	
	private int m_maxHealth;
	private float m_health;
	private float m_healthRegenRate;
	
	private int m_maxMana;
	private float m_manaRegenRate;
	private float m_mana;
	
	protected int m_exp;
	protected int m_expToLevel;
	
	private int m_level;

	private long m_attackSpeed;
	
	protected Animator m_weaponAnimator;
	
	protected long m_recoveryTime;
	protected long m_recoveryDuration;
	
	protected LinkedList<Attack> m_attacks;

	// =========================================================================
	//                            Constructors
	// =========================================================================
	
	public Creature(int strength, int defense, int intelligence, int health, int level, int exp) 
	{
		m_maxHealth = health;
		m_health = health;
				
		m_defense = defense;
		m_strength = strength;
		
		m_healthRegenRate = 0f;
		m_manaRegenRate = 0.001f;
				
		m_level = level;
		m_exp = exp;
		
		m_modifiedStr = m_strength;
		m_modifiedDef = m_defense;
		m_modifiedInt = m_intelligence;
		
		m_attackSpeed = 500;		

		m_recoveryTime = 1000L;
		m_recoveryDuration = 1000L;
		
		CalculateDamage();
		CalculateDamageReduction();
		CalculateMana();
	}
	
	public Creature(int level)
	{
		m_level = level;
		
		// starting statistics
		m_maxHealth = m_level * 10;
		m_health = m_maxHealth;
		
		m_strength = m_level * 5;
		m_defense = m_level * 5;
		m_intelligence = m_level * 5;
		
		m_exp = 0;
		m_expToLevel = m_level * 10;
		
		m_modifiedStr = m_strength;
		m_modifiedDef = m_defense;
		m_modifiedInt = m_intelligence;
		
		m_healthRegenRate = 0;
		m_manaRegenRate = 0.001f;
				
		m_attackSpeed = 500;	

		m_recoveryTime = 1000L;
		m_recoveryDuration = 1000L;
		
		CalculateDamage();
		CalculateDamageReduction();
		CalculateMana();
	}


	// =========================================================================
	//                            Public Methods
	// =========================================================================
	
	public long GetAttackSpeed()
	{
		return m_attackSpeed;
	}
	
	public int GetStrength()
	{
		return m_modifiedStr;
	}

	public int GetMaxHealth() 
	{
		return m_maxHealth;
	}

	public float GetHealth() 
	{
		return m_health;
	}

	public int GetMaxMana() 
	{
		return m_maxMana;
	}

	public int GetMana() 
	{
		return (int) m_mana;
	}

	public int GetExp() 
	{
		return m_exp;
	}

	public int GetIntelligence()
	{
		return m_modifiedInt;
	}
	
	public int GetExpToLevel()
	{
		return m_expToLevel;
	}

	public int GetLevel() 
	{
		return m_level;
	}
	
	public int GetDefense()
	{
		return m_modifiedDef;
	}
	
	public int GetDamage()
	{
		return m_damage;
	}

	public void SetStrength(int strength) 
	{
		m_modifiedStr -= m_strength;
		
		m_strength = strength;
		
		m_modifiedStr += m_strength;
		
		CalculateDamage();
	}

	public void SetMaxHealth(int m_maxHealth)
	{
		this.m_maxHealth = m_maxHealth;
	}

	public void SetMaxMana()
	{
		m_maxMana = 1000;
		FillMana();
	}
	
	public void SetIntelligence(int intelligence) 
	{
		m_modifiedInt -= m_intelligence;
		
		m_intelligence = intelligence;
		
		m_modifiedInt += m_intelligence;
		
		CalculateMana();
	}

	public void SetExpToLevel(int m_expToLevel) 
	{
		this.m_expToLevel = m_expToLevel;
	}

	public void SetLevel(int m_level) 
	{
		this.m_level = m_level;
	}
	
	public void SetDefense(int defense)
	{
		m_modifiedDef -= m_defense;
		
		m_defense = defense;
		
		m_modifiedDef += m_defense;
		
		CalculateDamageReduction();
	}
	
	public void FillHealth()
	{
		m_health = m_maxHealth;
	}
	
	public void FillMana()
	{
		m_mana = m_maxMana;
	}
	
	public void ModifyStrength(int strBuff)
	{
		m_modifiedStr += strBuff;
	}
	
	public void ModifyDefense(int defBuff)
	{
		m_modifiedDef += defBuff;
	}
	
	public void ModifyIntelligence(int intBuff)
	{
		m_modifiedInt += intBuff;
	}
	// =========================================================================
	//                            Overrides
	// =========================================================================
	
	@Override
	public void Paint(Graphics2D g)
	{
		if (m_currentImage == null)
		{
			return;
		}

		int x = (int)m_position.GetX();
		int y = (int)m_position.GetY();
		int width = m_currentImage.getWidth();
		int height = m_currentImage.getHeight();

		if (m_recoveryTime < m_recoveryDuration)
		{
			float alpha = (float)(Math.abs(Math.sin(m_recoveryTime * 2)));
			g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, alpha));
		}
		
		if (m_currentImage != null)
		{
			g.drawImage(m_currentImage, x, y, width, height, null, null);
		}
		
		//SetDebugInfo(m_animator.GetDebugInfo());
		
		PaintSpellEffects(g);
	}
	
	@Override
	public void Update(long elapsedGameTime) 
	{
		if (m_mana < m_maxMana)
		{
			m_mana += (m_manaRegenRate * elapsedGameTime);
		}
		
		if (m_mana > m_maxMana)
		{
			m_mana = m_maxMana;
		}
		
		if (m_health < m_maxHealth)
		{
			m_health += (m_healthRegenRate * elapsedGameTime);
		}
		
		if (m_health > m_maxHealth)
		{
			m_health = m_maxHealth;
		}
		
		LinkedList<Attack> toRemove = new LinkedList<Attack>();
		for(Attack atk : m_attacks)
		{
			atk.Update(elapsedGameTime);
			
			if (atk.IsExpired())
			{
				toRemove.add(atk);
			}
		}
		
		for(Attack atk : toRemove)
		{
			m_attacks.remove(atk);
		}
	}
	
	@Override
	protected void PaintSpellEffects(Graphics2D g)
	{
		for(Attack atk : m_attacks)
		{
			atk.Paint(g);
		}
		
		super.PaintSpellEffects(g);
	}
	
	public void TakeDamage(int damage, Rectangle damagingBox)
	{
		if (m_recoveryTime >= m_recoveryDuration)
		{
			damage -= m_damageReduction;
		
			if (damage <= 0)
			{
				// Always take 1 damage, minimum
				damage = 1;
			}
		
			m_health -= damage;
		}
	}

	// =========================================================================
	//                            Protected Methods
	// =========================================================================
	protected void DecreaseMana(int spellCost)
	{
		m_mana -= spellCost;
	}	
	
	protected void IncrementManaRegen()
	{
		m_manaRegenRate += 0.001f;
	}
	
	protected void IncrementHealthRegen()
	{
		m_healthRegenRate += 0.001f;
	}
	
	protected void IncrementAttackSpeed()
	{
		m_attackSpeed -= 25L;
	}
	
	protected void CalculateDamage()
	{
		m_damage = (10 + m_modifiedStr) / 10;
	}
	
	protected void CalculateDamageReduction()
	{
		m_damageReduction = (10 + m_modifiedDef) / 10;
	}
	
	protected void CalculateMana()
	{
		m_maxMana = (100 + m_modifiedInt) / 10;
	}
	
	protected void CalculateHealthRegenRate(int regenLevel)
	{
		m_healthRegenRate = (float) (regenLevel / 2f);
	}

	// =========================================================================
	//                            Private Methods
	// =========================================================================
	

}
