/**Copyright  2012 held by
 * Robert Williams
 * Robert Poland
 * Marc Hayes
 * Coen Smits
 * Justin Becker
 * All rights reserved.
 * 
 * 
 * This file is part of Bloodmoon.

    Bloodmoon is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bloodmoon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bloodmoon.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.AlphaComposite;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.Timer;

import javax.imageio.ImageIO;
import javax.swing.JFrame;


public class Game {//extends JFrame {
	/**
	 * 
	 */
	//private static final long serialVersionUID = 1L;
	
	private Timer m_timer;
	private GameEngine m_engine;
	private GameCanvas m_canvas;
	private KeyboardInput m_keyboard;
	private JFrame m_frame = null;

	public Game()
	{
		m_keyboard = new KeyboardInput(); // Keyboard polling
		m_timer = new Timer();
		m_engine = new GameEngine(1, m_keyboard);
		m_canvas = new GameCanvas(m_engine);
		
		m_engine.SetCanvas(m_canvas);
	}
	

	public void Start()
	{
		//1. Create the frame.
		m_frame = new JFrame("Blood Moon");

		//2. Optional: What happens when the frame closes?
		m_frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
				
		//3. Size the frame.
		// the 16/38 increase is for the windows frame size
		m_frame.setBounds(200, 200, m_engine.GetMapWidth(), m_engine.GetMapHeight()+20);
		//m_frame.pack();

		//4. Show it.
		m_frame.setVisible(true);
		m_frame.setResizable(false);
		
		//4.5 Add a key listener
		m_frame.addKeyListener(m_keyboard);
	
		//5. Create components and put them in the frame.
		m_frame.getContentPane().add(m_canvas, BorderLayout.CENTER);

		m_timer.schedule(m_engine, 0, 15);
	}
		
    
    static public BufferedImage LoadImage(String filePath)
    {
    	BufferedImage image = null;
		File imageFile = new File(filePath);
		
		try
		{
			image = ImageIO.read(imageFile);
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
    	
		return image;
    }

    static public void DarkenScreen(Graphics2D graphics)
    {
		graphics.setColor(Color.BLACK);
		graphics.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f));
		graphics.fillRect(0, 0, graphics.getClip().getBounds().width, graphics.getClip().getBounds().height);
		
		graphics.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 1.0f));
    }

}
