import java.util.LinkedList;

/**Copyright  2012 held by
 * Robert Williams
 * Robert Poland
 * Marc Hayes
 * Coen Smits
 * Justin Becker
 * All rights reserved.
 * 
 * 
 * This file is part of Bloodmoon.

    Bloodmoon is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bloodmoon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bloodmoon.  If not, see <http://www.gnu.org/licenses/>.
 */



public class InventoryItem
{	
	// Members
	private String m_name = "Inventory Item";
	private String m_description = "Just an Inventory Item";
	
	private int m_strengthBuff;
	private int m_defenseBuff;
	private int m_manaBuff;
	
	private ItemType m_itemType;
	
	private BodyPart m_bodyPart;
	
	public InventoryItem(int strength, int defense, int mana, String name, BodyPart part, ItemType type)
	{
		m_strengthBuff = strength;
		m_defenseBuff = defense;
		m_manaBuff = mana;
		
		m_name = name;
		
		m_bodyPart = part;
		
		m_itemType = type;
	}

	private static int RandomRange(int min, int max)
	{
		return min + (int)(Math.random() * ((max - min) + 1));
	}
	

	
	public static InventoryItem GenerateRandom()
	{	
		final class ItemInformation
		{
			public String Name;
			public BodyPart Part;
			public ItemType Item;
			public int StrengthMin;
			public int StrengthMax;
			public int DefenseMin;
			public int DefenseMax;
			public int ManaMin;
			public int ManaMax;
			
			public ItemInformation(String name,
					BodyPart part,
					ItemType item,
					int strengthMin,
					int strengthMax,
					int defenseMin,
					int defenseMax,
					int manaMin,
					int manaMax)
			{
				Name = name;
				Part = part;
				Item = item;
				StrengthMin = strengthMin;
				StrengthMax = strengthMax;
				DefenseMin = defenseMin;
				DefenseMax = defenseMax;
				ManaMin = manaMin;
				ManaMax = manaMax;
			}
		}
		
		LinkedList<ItemInformation> items = new LinkedList<ItemInformation>();
		items.add(new ItemInformation("Cloth Shirt", BodyPart.TORSO, ItemType.ARMOR_CLOTH, 0, 4, 1, 5, 0, 1));
		items.add(new ItemInformation("Cloth Shoes", BodyPart.FEET, ItemType.ARMOR_CLOTH, 0, 2, 1, 3, 0, 1));
		items.add(new ItemInformation("Cloth Belt", BodyPart.BELT, ItemType.ARMOR_CLOTH, 0, 1, 0, 1, 1, 4));
		items.add(new ItemInformation("Cloth Gloves", BodyPart.HANDS, ItemType.ARMOR_CLOTH, 1, 3, 1, 5, 0, 1));
		items.add(new ItemInformation("Cloth Hood", BodyPart.HEAD, ItemType.ARMOR_CLOTH, 0, 1, 1, 3, 0, 4));
		items.add(new ItemInformation("Cloth Pants", BodyPart.LEGS, ItemType.ARMOR_CLOTH, 0, 2, 1, 5, 0,1 ));
		items.add(new ItemInformation("Leather Shirt", BodyPart.TORSO, ItemType.ARMOR_LEATHER, 1, 5, 3, 8, 0, 3));
		items.add(new ItemInformation("Leather Shoes", BodyPart.FEET, ItemType.ARMOR_LEATHER, 1, 5, 3, 8, 0, 3));
		items.add(new ItemInformation("Leather Belt", BodyPart.BELT, ItemType.ARMOR_LEATHER, 1, 5, 3, 8, 3, 7));
		items.add(new ItemInformation("Leather Gloves", BodyPart.HANDS, ItemType.ARMOR_LEATHER, 1, 5, 3, 8, 0, 3));
		items.add(new ItemInformation("Leather Headgear", BodyPart.HEAD, ItemType.ARMOR_LEATHER, 1, 5, 3, 8, 0, 3));
		items.add(new ItemInformation("Leather Pants", BodyPart.LEGS, ItemType.ARMOR_LEATHER, 1, 5, 3, 8, 0, 3));
		items.add(new ItemInformation("Chain Shirt", BodyPart.TORSO, ItemType.ARMOR_CHAIN, 1, 5, 6, 10, 0, 3));
		items.add(new ItemInformation("Chain Shoes", BodyPart.FEET, ItemType.ARMOR_CHAIN, 1, 5, 3, 8, 0, 3));
		items.add(new ItemInformation("Chain Belt", BodyPart.BELT, ItemType.ARMOR_CHAIN, 1, 5, 3, 8, 5, 10));
		items.add(new ItemInformation("Chain Gloves", BodyPart.HANDS, ItemType.ARMOR_CHAIN, 1, 5, 3, 8, 0, 3));
		items.add(new ItemInformation("Chain Headgear", BodyPart.HEAD, ItemType.ARMOR_CHAIN, 1, 5, 3, 8, 0, 3));
		items.add(new ItemInformation("Chain Pants", BodyPart.LEGS, ItemType.ARMOR_CHAIN, 1, 5, 6, 15, 0, 3));
		items.add(new ItemInformation("Plate Armor", BodyPart.TORSO, ItemType.ARMOR_PLATE, 1, 5, 10, 15, 0, 3));
		items.add(new ItemInformation("Plate Shoes", BodyPart.FEET, ItemType.ARMOR_PLATE, 1, 5, 3, 8, 0, 3));
		items.add(new ItemInformation("Plate Belt", BodyPart.BELT, ItemType.ARMOR_PLATE, 1, 5, 3, 8, 7, 14));
		items.add(new ItemInformation("Plate Gloves", BodyPart.HANDS, ItemType.ARMOR_PLATE, 1, 5, 3, 8, 0, 3));
		items.add(new ItemInformation("Plate Helmet", BodyPart.HEAD, ItemType.ARMOR_PLATE, 1, 5, 3, 8, 0, 3));
		items.add(new ItemInformation("Plate Pants", BodyPart.LEGS, ItemType.ARMOR_PLATE, 1, 5, 10, 20, 0, 3));
		
		ItemInformation info = items.get(RandomRange(0, items.size() - 1));
		int strength = RandomRange(info.StrengthMin, info.StrengthMax);
		int defense = RandomRange(info.DefenseMin, info.DefenseMax);
		int mana = RandomRange(info.ManaMin, info.ManaMax);
		InventoryItem item = new InventoryItem(strength, defense, mana, info.Name, info.Part, info.Item);
		return item;
	}
	
	// Getters
	public String getName()
	{
		return m_name;
	}

	public int GetStrengthBuff()
	{
		return m_strengthBuff;
	}

	public int GetDefenseBuff() 
	{
		return m_defenseBuff;
	}

	public int GetManaBuff() 
	{
		return m_manaBuff;
	}
	
	public String GetDescription()
	{
		return m_description;
	}
	
	public BodyPart GetBodyPart()
	{
		return m_bodyPart;
	}
	
	public ItemType GetItemType()
	{
		return m_itemType;
	}
	
	// Setters
	
	public void SetDescription(String description)
	{
		m_description = description;
	}
}
