/**Copyright  2012 held by
 * Robert Williams
 * Robert Poland
 * Marc Hayes
 * Coen Smits
 * Justin Becker
 * All rights reserved.
 * 
 * 
 * This file is part of Bloodmoon.

    Bloodmoon is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bloodmoon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bloodmoon.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.event.KeyEvent;


public class InventoryOverlay extends ScrollOverlay
{
	// Members
	private PlayerCharacter m_player = null;
	private int m_selectedIndex;
	private InventoryItem m_selectedItem;
	
	public InventoryOverlay(GameEngine gameEngine, PlayerCharacter player)
	{
		super(gameEngine);

		m_player = player;
		m_selectedIndex = 0;
	}
	
	@Override
	public void Paint(Graphics2D graphics)
	{
		super.Paint(graphics);
		
		int y = m_y + m_scrollBarTopHeight;
		
		y += 10;
		graphics.drawString("INVENTORY", m_x + (m_scrollBarWidth / 2) - 20, y);

		y += 10;
		
		int index = 0;
		// Do your custom inventory rendering in here
		for( InventoryItem inventoryItem : m_player.GetInventory())
		{
			y += 20;

			graphics.setColor(Color.BLACK);
			graphics.drawString(inventoryItem.getName(), m_x + 20, y);
			
			if (index == m_selectedIndex)
			{
				// Highlight this one
				graphics.setColor(Color.BLUE);
				graphics.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.25f));
				graphics.fillRect(m_x+10, y-13, 200, 15);
			
				graphics.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 1.0f));
				graphics.drawRect(m_x+10, y-13, 200, 15);
				
				m_selectedItem = inventoryItem;
			}
			
			index++;
		}
		
		if(m_player.GetInventory().isEmpty())
		{
			y += 20;
			graphics.setColor(Color.BLACK);
			graphics.drawString("-EMPTY-", m_x + 20, y);
			m_selectedItem = null;
		}
		
		// Print equipped items on the right
		graphics.setColor(Color.BLACK);
		
		y = m_y + m_scrollBarTopHeight + 40;
		int x = m_x + (m_scrollBarWidth / 2);

		//
		int newStr = 0;
		int newInt = 0;
		int newDef = 0;
		// inventory has items
		if(m_selectedItem != null)
		{
			newStr = m_player.GetStrength() + m_selectedItem.GetStrengthBuff();
			newInt = m_player.GetIntelligence() + m_selectedItem.GetManaBuff();
			newDef = m_player.GetDefense() + m_selectedItem.GetDefenseBuff();
		}
		else //inventory is empty, so dont compare to anything
		{			
			newStr = m_player.GetStrength();
			newInt = m_player.GetIntelligence();
			newDef = m_player.GetDefense();
		}			

		PrintStat(graphics, "Strength", m_player.GetStrength(), newStr, x + 20, y);
		
		y += 20;
		PrintStat(graphics, "Defense", m_player.GetDefense(), newDef, x + 20, y);
				
		y += 20;
		PrintStat(graphics, "Intelligence", m_player.GetIntelligence(), newInt, x + 20, y);

		y += 20;
		graphics.drawString("Health: " + m_player.GetHealth() + " / " + m_player.GetMaxHealth(), x + 20, y);
		
		y += 20;
		graphics.drawString("Mana: " + m_player.GetMana() + " / " + m_player.GetMaxMana(), x + 20, y);
		
		y += 40;
		for(BodyPart part : BodyPart.values())
		{
			y += 20;
			
			InventoryItem equiped = m_player.GetEquipedItems().get(part);
			
			if (equiped == null)
			{
				graphics.drawString(part.toString() + ": ", x + 20, y);
			}
			else
			{
				graphics.drawString(part.toString() + ": " + equiped.getName(), x + 20, y);
			}
		}
	}
	
	private void PrintStat(Graphics2D g, String statName, int oldStat, int newStat, int x, int y)
	{
		if (newStat > oldStat)
		{
			g.setColor(Color.GREEN.darker().darker());
			g.drawString(statName + ": " + newStat + " ( +" + (newStat - oldStat) + " )", x, y);
			g.setColor(Color.BLACK);
		}
		else if (newStat < oldStat)
		{
			g.setColor(Color.RED);
			g.drawString(statName + ": " + newStat + " ( -" + (oldStat - newStat) + " )", x, y);
			g.setColor(Color.BLACK);
		}
		else
		{
			g.setColor(Color.BLACK);
			g.drawString(statName + ": " + oldStat, x, y);
		}
	}
	
	@Override
	public void Update(long elapsedGameTime, KeyboardInput keyboard)
	{
		super.Update(elapsedGameTime, keyboard);
		
		if( keyboard.keyDownOnce(KeyEvent.VK_I) || 
			keyboard.keyDownOnce(KeyEvent.VK_TAB))
		{
			m_gameEngine.setGameMode();
		}
		else if (keyboard.keyDownOnce(KeyEvent.VK_S) ||
				keyboard.keyDownOnce(KeyEvent.VK_KP_DOWN) ||
				keyboard.keyDownOnce(KeyEvent.VK_DOWN))
		{			
			// Move menu button down			
			if( m_selectedIndex < m_player.GetInventory().size()-1 )
			{
				m_selectedIndex++;
			}
		}
		else if (keyboard.keyDownOnce(KeyEvent.VK_W) ||
					 keyboard.keyDownOnce(KeyEvent.VK_KP_UP) ||
					 keyboard.keyDownOnce(KeyEvent.VK_UP))
		{
			// Move menu button up
			if( m_selectedIndex > 0 )
			{
				m_selectedIndex--;
			}
		}
		else if (keyboard.keyDownOnce(KeyEvent.VK_ENTER))
		{
			if(m_selectedItem == null)
			{
				return;
			}
			
			m_player.Equip(m_selectedItem);
			
			m_selectedIndex--;
			if (m_selectedIndex < 0)
			{
				m_selectedIndex = 0;
			}
		}
	}
}
