/**Copyright  2012 held by
 * Robert Williams
 * Robert Poland
 * Marc Hayes
 * Coen Smits
 * Justin Becker
 * All rights reserved.
 * 
 * 
 * This file is part of Bloodmoon.

    Bloodmoon is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bloodmoon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bloodmoon.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.Graphics2D;
import java.awt.event.KeyEvent;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.List;

public class Menu
{	
	// Members
	private GameEngine m_gameEngine = null;
	private List<MenuButton> m_menuButtonList;
	private MenuButton m_selectedButton = null;
	private BufferedImage m_buttonImage = null;
	private BufferedImage m_buttonImageSelected = null;
	
    public Menu(GameEngine gameEngine)
    {
    	m_buttonImage = Game.LoadImage(".\\Resources\\Images\\Menu\\Scroll.png");
    	m_buttonImageSelected = Game.LoadImage(".\\Resources\\Images\\Menu\\ScrollSelected.png");
    	
    	m_gameEngine = gameEngine;
    	
    	m_menuButtonList = new ArrayList<MenuButton>();

    	MenuButton startButton = AddButton((13 * 32) + 16, (8 * 32), MenuAction.START, "Start");
    	AddButton((13 * 32) + 16, (9 * 32), MenuAction.HELP, "Help");
    	AddButton((13 * 32) + 16, (10 * 32), MenuAction.QUIT, "Quit");
    	    	
    	m_selectedButton = startButton;
    }
    
    private MenuButton AddButton(int x, int y, MenuAction action, String text)
    {
    	MenuButton button = new MenuButton(x,y, action, text);
    	
    	button.setImage(m_buttonImage);
    	button.setImageSelected(m_buttonImageSelected);
    	
    	m_menuButtonList.add(button);

    	return button;
    }

	public void Paint(Graphics2D graphics)
	{
		// Transparent background
		Game.DarkenScreen(graphics);

		for(MenuButton menuButton :  m_menuButtonList)
		{
			boolean isSelected = m_selectedButton == menuButton;
			
			menuButton.Paint(graphics, isSelected);
		}
    }
	
	public void Update(long elapsedGameTime, KeyboardInput keyboard)
	{	
		int currentIndex = m_menuButtonList.indexOf(m_selectedButton);

		if (keyboard.keyDownOnce(KeyEvent.VK_S) ||
			keyboard.keyDownOnce(KeyEvent.VK_KP_DOWN) ||
			keyboard.keyDownOnce(KeyEvent.VK_DOWN))
		{			
			// Move menu button down			
			if( currentIndex < m_menuButtonList.size()-1 )
			{
				m_selectedButton = m_menuButtonList.get(currentIndex+1);
			}
		}
		else if (keyboard.keyDownOnce(KeyEvent.VK_W) ||
				 keyboard.keyDownOnce(KeyEvent.VK_KP_UP) ||
				 keyboard.keyDownOnce(KeyEvent.VK_UP))
		{
			// Move menu button up
			if( currentIndex > 0 )
			{
				m_selectedButton = m_menuButtonList.get(currentIndex-1);
			}
		}
		else if( keyboard.keyDownOnce(KeyEvent.VK_ESCAPE))
		{
			m_gameEngine.setGameMode();
		}
		else if( keyboard.keyDownOnce(KeyEvent.VK_ENTER))
		{
			// Run command
			switch(m_selectedButton.getMenuAction())
			{
				case START:
					m_gameEngine.setGameMode();
					break;
				case HELP:
					m_gameEngine.ShowHelp();
					break;
				case QUIT:
					System.exit(0);
					break;	
				case NONE:
					break;
			}
		}
	}
}
