/**Copyright  2012 held by
 * Robert Williams
 * Robert Poland
 * Marc Hayes
 * Coen Smits
 * Justin Becker
 * All rights reserved.
 * 
 * 
 * This file is part of Bloodmoon.

    Bloodmoon is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bloodmoon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bloodmoon.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.Rectangle;
import java.util.LinkedList;

public abstract class Monster extends Creature
{

	// =========================================================================
	//                           Members
	// =========================================================================
	private int m_sustain = 0;
	private Direction m_wanderDirection = Direction.STANDSTILL;
	
	private int m_reach = 150; // Reach in pixels
	private boolean m_aggressoMode = false;
	private PlayerCharacter m_playerCharacter = null;
	
	private Vector2D m_lastPosition = new Vector2D(0,0);
	
	protected boolean m_canShootAtPlayer = false;
	
	// Getters
	
	// Setters
	public void setAggressoMode(boolean value)
	{
		m_aggressoMode = value;
	}
	
	// =========================================================================
	//                           Constructors
	// =========================================================================
	public Monster(int monsterLevel)
	{
		// Invoke base (super) class constructor
		super(monsterLevel);
		m_exp = monsterLevel * 5;
		
		// AI defaults
		m_sustain = 0;
		m_wanderDirection = Direction.STANDSTILL;
		
		// Flying monster so true
		m_animationIsConstant = true;
		
		m_attacks = new LinkedList<Attack>();
	}


	// =========================================================================
	//                           Overrides
	// =========================================================================
	@Override
	public void Update(long elapsedGameTime) {
		// look for nearby characters in aggresso range
		
		// if aggresso is enabled, chase player
		if( m_aggressoMode )
		{
			float x = 0.0f;
			float y = 0.0f;
			float randomVar = 0.1f; // velocity?
			
			Vector2D playerPos = new Vector2D(m_playerCharacter.m_boundingBox.x + m_playerCharacter.m_boundingBox.width,
											  m_playerCharacter.m_boundingBox.y + m_playerCharacter.m_boundingBox.height);
			Vector2D monsterPos = new Vector2D(this.m_boundingBox.x + this.m_boundingBox.width,
											   this.m_boundingBox.y + this.m_boundingBox.height);
			
			// Find the location of the player
			Direction direction = Direction.STANDSTILL;
			
			// Ugly indeed.. but what do you do without a proper vector
			if( playerPos.GetX() > monsterPos.GetX() )
			{
				x = randomVar;
				direction = Direction.EAST;
				
				if(playerPos.GetY() > monsterPos.GetY() )
				{
					y = randomVar;
				}
				else if( playerPos.GetY() < monsterPos.GetY() )
				{
					y = -1.0f * randomVar;
				}
			}
			else if( playerPos.GetX() < monsterPos.GetX() )
			{
				x = -1.0f * randomVar;
				direction = Direction.WEST;
				
				if(playerPos.GetY() > monsterPos.GetY() )
				{
					y = randomVar;
				}
				else if( playerPos.GetY() < monsterPos.GetY() )
				{
					y = -1.0f * randomVar;
				}
			}
			else
			{
				if(playerPos.GetY() > monsterPos.GetY() )
				{
					y = -1.0f * randomVar;
					direction = Direction.NORTH;
				}
				else if( playerPos.GetY() < monsterPos.GetY() )
				{
					y = randomVar;
					direction = Direction.SOUTH;
				}
			}
			
			m_velocity.Set(x, y);
			m_animator.BeginNewAction(direction, Action.Idle, 1000, true);
		}
		else
		{
			// if passive, wander around		
			Wander(elapsedGameTime);	
		}
			
		// rotating image for movement
		m_currentImage = m_animator.CalculateNextFrameSinusoidal();
		
		// update position
		m_lastPosition.Set(m_position.GetX(), m_position.GetY());
		UpdatePosition(elapsedGameTime);
		
		if (Global.rand.nextInt(1000) < 10)
		{
			ShootAtPlayer();
		}
		
		super.Update(elapsedGameTime);
	}

	@Override
	protected void InitializeStartingVectors() 
	{
		// positional vectors
		m_acceleration = new Vector2D(0.0f, 0.0f);
		m_velocity = new Vector2D(0.0f, 0.0f);
		m_position = new Vector2D((float)(Global.rand.nextFloat() * 650.0f + 150.0f), (float)(Global.rand.nextFloat() * 300.0f + 150.0f));		
	}
		
	// =========================================================================
	//                           Public Methods
	// =========================================================================
	
	public void CheckCollisions(GameObject obj)
	{
		Rectangle objBox = obj.GetBoundingBox();
		
		if(objBox == null)
		{
			return;
		}
		
		if (m_boundingBox.intersects(objBox))
		{
			if (obj instanceof Wall)
			{
				m_position.Set(m_lastPosition.GetX(), m_lastPosition.GetY());
			}
		}
		
		if (obj instanceof PlayerCharacter)
		{
			for(Attack atk : m_attacks)
			{
				if (atk.GetBoundingBox().intersects(objBox))
				{
					((PlayerCharacter)obj).TakeDamage(atk.GetDamage(), atk.GetBoundingBox());
					((PlayerCharacter)obj).KnockBack(atk.GetBoundingBox());
					
					atk.SetExpired();
					break;
				}
			}
		}
	}
			
			
	public boolean IsClose(PlayerCharacter player)
	{
		// Performance is better when not using Pythagoras
		Vector2D playerPos = new Vector2D(player.m_boundingBox.x + player.m_boundingBox.width,
										  player.m_boundingBox.y + player.m_boundingBox.height);
		Vector2D monsterPos = new Vector2D(this.m_boundingBox.x + this.m_boundingBox.width,
					   					   this.m_boundingBox.y + this.m_boundingBox.height);
		Vector2D resultVector = playerPos.Subtract(monsterPos);
		
		if( Math.abs(resultVector.GetX()) <= m_reach ||
			Math.abs(resultVector.GetY()) <= m_reach )
		{
			if( playerPos.Distance(monsterPos) <= (double) m_reach)
			{
				// Make it hunt the player
				m_playerCharacter = player;
				return true;
			}
		}
		
		m_playerCharacter = null;
		return false;
	}
	
	public boolean MonsterDead()
	{
		return (GetHealth() < 1);
	}
	
	public GameObject GetLoot()
	{
		if (MonsterDead())
		{
			return RandomTreasure();
		}
		
		return null;
	}
	
	public int GetExp()
	{
		return m_exp;
	}

	// =========================================================================
	//                           Abstract Methods
	// =========================================================================
	
	protected abstract GameObject RandomTreasure();
	
	// =========================================================================
	//                           Protected Methods
	// =========================================================================
	protected void Wander(long elapsedGameTime)
	{
		float accel = 0.01f;
		
		// idle sustain management
		if(m_sustain > 0)
		{
			m_sustain -= 1;
		} 
		else
		{
			// choose new action if last action was standing still
			if(m_wanderDirection == Direction.STANDSTILL)
			{
				m_wanderDirection = Direction.values()[Global.rand.nextInt(Direction.values().length - 1)];
				m_sustain = Global.rand.nextInt(30);
			}
			else
			{
				// idle if last action was movement
				m_acceleration.AddX(accel);
				m_wanderDirection = Direction.STANDSTILL;
				m_sustain = Global.rand.nextInt(90) + 30;
			}
		}
		
		// set vectors for chosen actions
		m_usesAccel = false;
		m_usesDecel = false;
		m_usesBounding = true;
		
		if( m_wanderDirection == Direction.STANDSTILL )
		{
			m_velocity.Set(0.0f, 0.0f);		
		}
		else if( m_wanderDirection == Direction.NORTH || m_wanderDirection == Direction.NORTHEAST )
		{
			// Walk north
			m_velocity.Set(0.0f, -0.1f);
			m_animator.BeginNewAction(Direction.NORTH, Action.Idle, 1000, true);
		}
		else if( m_wanderDirection == Direction.EAST || m_wanderDirection == Direction.SOUTHEAST )
		{
			// Walk east
			m_velocity.Set(0.1f, 0.0f);
			m_animator.BeginNewAction(Direction.EAST, Action.Idle, 1000, true);
		}
		else if( m_wanderDirection == Direction.SOUTH || m_wanderDirection == Direction.SOUTHWEST )
		{
			// Walk south
			m_velocity.Set(0.0f, 0.1f);
			m_animator.BeginNewAction(Direction.SOUTH, Action.Idle, 1000, true);
		}
		else if( m_wanderDirection == Direction.WEST || m_wanderDirection == Direction.NORTHWEST )
		{
			// Walk west
			m_velocity.Set(-0.1f, 0.0f);
			m_animator.BeginNewAction(Direction.WEST, Action.Idle, 1000, true);
		}
	}

	protected void ShootAtPlayer()
	{
		if (m_playerCharacter != null)
		{
			// Performance is better when not using Pythagoras
			Vector2D playerPos = new Vector2D(m_playerCharacter.m_boundingBox.x + m_playerCharacter.m_boundingBox.width,
					m_playerCharacter.m_boundingBox.y + m_playerCharacter.m_boundingBox.height);
			
			Vector2D monsterPos = new Vector2D(this.m_boundingBox.x + this.m_boundingBox.width,
					   this.m_boundingBox.y + this.m_boundingBox.height);
			
			Vector2D resultVector = playerPos.Subtract(monsterPos);
			
			float velocityX = resultVector.GetX() * 0.001f;
			float velocityY = resultVector.GetY() * 0.001f;

			m_attacks.add(new Fireball(monsterPos, velocityX, velocityY, GetDamage(), 1000, 0));
		}
	}

	protected void SpawnMonster(Vector2D acceleration, Vector2D velocity, Vector2D position) 
	{
		// positional vectors
		m_acceleration = new Vector2D(acceleration.GetX(), acceleration.GetY());
		m_velocity = new Vector2D(velocity.GetX(), velocity.GetY());
		m_position = new Vector2D(position.GetX(), position.GetY());		
	}
}