/**Copyright  2012 held by
 * Robert Williams
 * Robert Poland
 * Marc Hayes
 * Coen Smits
 * Justin Becker
 * All rights reserved.
 * 
 * 
 * This file is part of Bloodmoon.

    Bloodmoon is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bloodmoon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bloodmoon.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.AlphaComposite;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.event.KeyEvent;
import java.awt.image.BufferedImage;
import java.util.HashMap;
import java.util.LinkedList;


public class PlayerCharacter extends Creature
{	
	protected int m_money;
	private HashMap<BodyPart, BufferedImage> m_equipmentImages;
	private HashMap<BodyPart, InventoryItem> m_equipedItems;
	protected Vector2D m_lastPosition = new Vector2D(0, 0);
	
	protected int m_skillPoints = 0;
	
	// Skills
	private HashMap<PlayerSkill, Integer> m_skills;
		
	// Fireball
	protected int m_fireballCost;
	protected long m_fireballDuration;
	protected float m_fireballSpeed = 0.45f;
	protected int m_fireballDamage;
	
	// Heavy Strike
	protected int m_heavyStrikeCost;
	protected long m_heavyStrikeDuration;
	protected float m_heavyStrikeSpeed;
	protected int m_heavyStrikeDamage;
	
	// Heavy Strike
	protected int m_multiShotCost;
	protected long m_multiShotDuration;
	protected float m_multiShotSpeed = 0.45f;
	protected int m_multiShotDamage;
	
	// Melee weapon
	private WeaponType m_currentWeapon = WeaponType.Dagger;
	
	// Inventory
	protected LinkedList<InventoryItem> m_inventory;
	
	// Getters
	public LinkedList<InventoryItem> GetInventory()
	{
		return m_inventory;
	}
	
	public HashMap<BodyPart, InventoryItem> GetEquipedItems()
	{
		return m_equipedItems;
	}
	
	public int GetSkillPoints()
	{
		return m_skillPoints;
	}
	
	public HashMap<PlayerSkill, Integer> GetPlayerSkills()
	{
		return m_skills;
	}
	
	// Setters
	
	
	// =========================================================================
	//                           Constructors
	// =========================================================================

	public PlayerCharacter()
	{
		// Invoke base (super) class constructor
		super(1);
				
		m_inventory = new LinkedList<InventoryItem>();
		
		//load in character	
		m_attacks = new LinkedList<Attack>();
		
		// statistics
		m_money = 0;

		// other
		m_boundingBox.width /= 2;
		m_boundingBox.height /= 1.5;

		m_boundingBoxOffSetX = m_boundingBox.width / 2;
		m_boundingBoxOffSetY = m_boundingBox.height / 2;
		
		m_equipmentImages = new HashMap<BodyPart, BufferedImage>();

		m_equipedItems = new HashMap<BodyPart, InventoryItem>();
		
		for(BodyPart part : BodyPart.values())
		{
			m_equipmentImages.put(part, null);
			m_equipedItems.put(part, null);
		}
		
		m_skills = new HashMap<PlayerSkill, Integer>();
		
		int startSkill = 0;
		m_skillPoints = 1;
		for(PlayerSkill skill : PlayerSkill.values())
		{
			m_skills.put(skill, startSkill);
		}
		
		m_recoveryTime = 1000L;
		m_recoveryDuration = 1000L;
		
		FillMana();
		
		Equip(new InventoryItem(0, 0, 0, "Shirt", BodyPart.TORSO, ItemType.ARMOR_CLOTH));
		Equip(new InventoryItem(0, 0, 0, "Pants", BodyPart.LEGS, ItemType.ARMOR_CLOTH));
	}
	
	// =========================================================================
	//                           Private Methods
	// =========================================================================

	private void LevelUp()
	{
		// update exp
		m_exp -= m_expToLevel;
		SetLevel(GetLevel() + 1);
		m_expToLevel = GetLevel() * 10;
		
		// update health
		SetMaxHealth(GetLevel() + 4 * 10);
		
		// update stats
		SetIntelligence(GetIntelligence() + 3);
		SetStrength(GetStrength() + 3);
		SetDefense(GetDefense() + 3);
		
		// Grant skill point
		m_skillPoints++;
		
		FillMana();
		FillHealth();
	}
	
	// =========================================================================
	//                            Public Methods
	// =========================================================================
	
	/**
	 * Equip an item to the character
	 * @param item
	 */
	public void Equip(InventoryItem item)
	{
		BodyPart part = item.GetBodyPart();
		ItemType type = item.GetItemType();
		
		if (type != null && type != ItemType.CONSUMABLE)
		{
			if (part != null)
			{
				InventoryItem oldItem = m_equipedItems.get(part);
				m_equipedItems.put(part, item);
			
				if (oldItem != null)
				{
					m_inventory.add(oldItem);
				}

				m_inventory.remove(item);
			
				if (oldItem != null)
				{
					ModifyStrength (-1 * oldItem.GetStrengthBuff());
					ModifyIntelligence(-1 * oldItem.GetManaBuff());
					ModifyDefense(-1 * oldItem.GetDefenseBuff());
				}
			
				ModifyStrength (item.GetStrengthBuff());
				ModifyIntelligence(item.GetManaBuff());
				ModifyDefense(item.GetDefenseBuff());
						
				CalculateDamage();
				CalculateDamageReduction();
				CalculateMana();
			
				LoadNewTextures(part, type);
			}
			else if (type == ItemType.CONSUMABLE)
			{
				// TODO implement consumables
			}			
		}		
	}
	
	public void LearnSkill(PlayerSkill skill)
	{
		if (m_skillPoints > 0)
		{
			Integer skillLevel = m_skills.get(skill);
			
			skillLevel ++;
			m_skillPoints --;
			
			m_skills.put(skill,  skillLevel);
			
			if (skill == PlayerSkill.Health_Passive)
			{
				IncrementHealthRegen();
			}
			else if (skill == PlayerSkill.Mana_Regen)
			{
				IncrementManaRegen();
			}
			else if (skill == PlayerSkill.Attack_Speed)
			{
				IncrementAttackSpeed();
			}
			else if (skill == PlayerSkill.Fireball)
			{
				CalculateFireBall(skillLevel);
			}
			else if (skill == PlayerSkill.Heavy_Strike)
			{
				CalculateHeavyStrike(skillLevel);
			}
			else if (skill == PlayerSkill.Multishot)
			{
				CalculateMultiShot(skillLevel);
			}
		}
	}
	
	private void CalculateFireBall(Integer skillLevel)
	{
		// skills
		m_fireballCost = 3 + (2 * skillLevel);
		m_fireballDuration = 750 - (50 * skillLevel.intValue());
		m_fireballSpeed = 0.45f;
		m_fireballDamage = (int) (GetIntelligence() * (1.5 * skillLevel.intValue()));
	}
	
	private void CalculateHeavyStrike(Integer skillLevel)
	{
		// skills
		m_heavyStrikeCost = 3 + (2 * skillLevel);
		m_heavyStrikeDuration = 1000 - (50 * skillLevel.intValue());
		m_heavyStrikeDamage = (int) (GetDamage() * (1.5 * skillLevel.intValue()));
	}
	
	private void CalculateMultiShot(Integer skillLevel)
	{
		// skills
		m_multiShotCost = 3 + (2 * skillLevel);
		m_multiShotDuration = 1000 - (50 * skillLevel.intValue());
		m_multiShotSpeed = 0.45f;
		m_multiShotDamage = (int) (GetIntelligence() * (1.5 * skillLevel.intValue()));
	}
	/**
	 * Update the state of the player, based on time and keyboard input
	 * @param elapsedGameTime
	 * @param keyboard
	 */
	public void Update(long elapsedGameTime, KeyboardInput keyboard, LinkedList<GameObject> currentGameObjects)
	{
		Direction currentDirection = m_animator.GetCurrentDirection();
		Action currentAction = m_animator.GetCurrentAction();
		
		if (currentAction == Action.Idle || currentAction == Action.Walk)
		{
			CheckKeyboardState(keyboard, currentDirection, currentAction);
		}
		
		m_usesAccel = true;
		m_usesDecel = true;
		m_deceleration = 0.1f;
		m_usesBounding = true;
		
		m_lastPosition.Set(m_position.GetX(), m_position.GetY());
		UpdatePosition(elapsedGameTime);
		
		m_currentImage = m_animator.CalculateNextFrameLinear(elapsedGameTime);
		
		for(BodyPart part : BodyPart.values())
		{
			m_equipmentImages.put(part, m_animator.CalculateNextFrameLinear(part));
		}
		
		if (m_recoveryTime < m_recoveryDuration)
		{
			m_recoveryTime += elapsedGameTime;
		}
	}	
	
	public void CheckCollisions(GameObject obj)
	{
		Rectangle objBox = obj.GetBoundingBox();
		Rectangle attackBox;
		
		if(objBox == null)
		{
			return;
		}
		
		if (m_boundingBox.intersects(objBox))
		{
			if (obj instanceof Wall)
			{
				m_position.Set(m_lastPosition.GetX(), m_lastPosition.GetY());
			}
			else if (obj instanceof Coin)
			{
				m_money += ((Coin)obj).GetValue();
				obj.SetExpired();		
			}
			else if (obj instanceof Item)
			{
				Item item = (Item)obj;
				
				if (!item.IsExpired())
				{
					m_inventory.add(item.GetInventoryItem());
				}
				
				item.SetExpired();
			}
			else if (obj instanceof Monster && m_recoveryTime >= m_recoveryDuration)
			{
				TakeDamage(((Monster)obj).GetDamage(), objBox);
				
				KnockBack(objBox);
			}
			else if (obj instanceof MapZoneBlock)
			{
				((MapZoneBlock)obj).DoHitAction();
			}
		}
		
		for(Attack atk : m_attacks)
		{
			attackBox = atk.GetBoundingBox();
			
			if(attackBox == null)
			{
				return;
			}
			
			if (attackBox.intersects(objBox))
			{
				if (obj instanceof Monster)
				{
					((Monster)obj).TakeDamage(atk.GetDamage(), attackBox);
					
					atk.SetExpired();
				}
			}
		}
	}
	
	public void KnockBack(Rectangle objBox)
	{
		if (m_recoveryTime >= m_recoveryDuration)
		{
			m_recoveryTime = 0;
		
			int enemyCenterX = objBox.x + (objBox.width / 2);
			int playerCenterX = m_boundingBox.x + (m_boundingBox.width / 2);
		
			int enemyCenterY = objBox.y + (objBox.height / 2);
			int playerCenterY = m_boundingBox.y + (m_boundingBox.height / 2);
						
			m_acceleration.Set(0.0075f * (playerCenterX - enemyCenterX), 
								0.0075f * (playerCenterY - enemyCenterY));
		}
	}
	// ==============================================
	//                 Overrides
	// ==============================================
	
	@Override
	protected void PaintSpellEffects(Graphics2D g)
	{
//		for(Attack atk : m_attacks)
//		{
//			atk.Paint(g);
//		}
		
		if (m_skillPoints > 0)
		{
			//TODO: force update of skills here
//			int posX = 200;
//			int posY = 100;
//			g.drawString("Press 'u' to spend skill points", posX, posY);
		}
		
		g.drawString("" + m_money, 760, 25);
		
		super.PaintSpellEffects(g);
	}
	
	@Override
	protected void InitializeSprites()
	{
		// Create a new animator (even though the base created a simple one)
		m_animator = new Animator(true);
		
		// Set animation frame data
		int frameHeight = 64;
		int frameWidth = 64;
				
		// Order directions for looping
		LinkedList<Direction> directionsInOrder = new LinkedList<Direction>();
		directionsInOrder.add(Direction.NORTH);
		directionsInOrder.add(Direction.WEST);
		directionsInOrder.add(Direction.SOUTH);
		directionsInOrder.add(Direction.EAST);
		
		// =====================================================
		//                        Body
		// =====================================================
			
		// Walking
		m_animator.LoadSpriteSheet(".\\Resources\\Images\\walkcycle\\BODY_male.png");		
		m_animator.AddAnimations(Action.Walk, directionsInOrder, frameHeight, frameWidth, 9);

		// Idle
		m_animator.AddAnimations(Action.Idle, directionsInOrder, frameHeight, frameWidth, 1);

		// Casting
		m_animator.LoadSpriteSheet(".\\Resources\\Images\\spellcast\\BODY_male.png");		
		m_animator.AddAnimations(Action.Spell, directionsInOrder, frameHeight, frameWidth, 7);

		// Slashing
		m_animator.LoadSpriteSheet(".\\Resources\\Images\\slash\\BODY_human.png");		
		m_animator.AddAnimations(Action.Slash, directionsInOrder, frameHeight, frameWidth, 6);
		
		// Thrusting
		m_animator.LoadSpriteSheet(".\\Resources\\Images\\thrust\\BODY_animation.png");		
		m_animator.AddAnimations(Action.Thrust, directionsInOrder, frameHeight, frameWidth, 8);
		
		m_animator.LoadSpriteSheet(".\\Resources\\Images\\bow\\BODY_animation.png");
		m_animator.AddAnimations(Action.Bow, directionsInOrder, frameHeight, frameWidth, 13);
		
		// Death
		m_animator.LoadSpriteSheet(".\\Resources\\Images\\hurt\\BODY_male.png");		
		m_animator.AddAnimation(Action.Fall, Direction.SOUTH, 0, 0, frameHeight, frameWidth, 6);
		
		// =====================================================
		//                  Default Equipment
		// =====================================================
			
		LoadAnimations(directionsInOrder, frameHeight, frameWidth, "HEAD_hair_blonde.png", BodyPart.HEAD);
		
		// Set initial state
		m_animator.BeginNewAction(Direction.EAST, Action.Idle, 500, true);
		
		m_animator.Finish();
		
		m_currentImage = m_animator.CalculateNextFrameLinear(0);
	}
	
	@Override
	protected void InitializeStartingVectors() 
	{
		m_acceleration = new Vector2D(0.0f, 0.0f);
		m_velocity = new Vector2D(0.0f, 0.0f);
		m_position = new Vector2D(100, 100);		
	}
	
	@Override
	public void Paint(Graphics2D g)
	{
		// can't use the base class, 'cause we have to draw body, then armor
		//super.Paint(g);		
		if (m_currentImage == null)
		{
			return;
		}

		int x = (int)m_position.GetX();
		int y = (int)m_position.GetY();
		int width = m_currentImage.getWidth();
		int height = m_currentImage.getHeight();

		if (m_recoveryTime < m_recoveryDuration)
		{
			float alpha = (float)(Math.abs(Math.sin(m_recoveryTime * 2)));
			g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, alpha));
		}
		
		if (m_currentImage != null)
		{
			g.drawImage(m_currentImage, x, y, width, height, null, null);
		}		
		
		BufferedImage equipment = null;
		
		for(BodyPart part : BodyPart.values())
		{
			equipment = m_equipmentImages.get(part);
		
			if (equipment != null)
			{
				g.drawImage(equipment, x, y, width, height, null, null);
			}
		}

		g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 1.0f));
		
		PaintSpellEffects(g);
	}
	
	
	// =========================================================================
	//                           Protected Methods
	// =========================================================================
	
	protected void getExp(Monster enemy)
	{
		// add in exp
		m_exp += enemy.GetExp();
		
		// check for level up
		if(m_exp >= (GetLevel() * 10))
		{
			LevelUp();
		}
	}

	// =========================================================================
	//                           Private Methods
	// =========================================================================
	
	private void CheckKeyboardState(KeyboardInput keyboard, Direction currentDirection, Action currentAction)
	{		
		boolean newAction = false;
		long newActionDuration = 1000;
		boolean loopNewAction = true;
		
		// Debug
		// TODO remove before release
		if (keyboard.keyDownOnce(KeyEvent.VK_G))
		{
			if (m_currentWeapon == WeaponType.Dagger)
			{
				m_currentWeapon = WeaponType.Bow;
			} 
			else if (m_currentWeapon == WeaponType.Bow)
			{
				m_currentWeapon = WeaponType.Spear; 
			}
			else if (m_currentWeapon == WeaponType.Spear)
			{
				m_currentWeapon = WeaponType.Staff;
			}
			else if (m_currentWeapon == WeaponType.Staff)
			{
				m_currentWeapon = WeaponType.Dagger; 
			}
		}
		
		// fire ball attack
		if(keyboard.keyDownOnce(KeyEvent.VK_J) && 
				m_skills.get(PlayerSkill.Fireball) > 0 && 
				(GetMana() >= m_fireballCost))
		{
			DecreaseMana(m_fireballCost);
			
			float velocityX = 0.0f;
			float velocityY = 0.0f;					
			
			if (currentDirection == Direction.NORTH)
			{
				velocityY = -1f * m_fireballSpeed;
			}
			else if (currentDirection == Direction.WEST)
			{
				velocityX = -1f * m_fireballSpeed;
			}
			else if (currentDirection == Direction.SOUTH)
			{
				velocityY = m_fireballSpeed;
			}
			else if (currentDirection == Direction.EAST)
			{
				velocityX = m_fireballSpeed;
			}
			
			currentAction = Action.Spell;
			newActionDuration = m_fireballDuration;
			loopNewAction = false;
			newAction = true;
			
			Vector2D projectilePosition = new Vector2D(m_position.GetX() + m_currentImage.getWidth()/2 - 10, m_position.GetY() + m_currentImage.getHeight()/2);
			m_attacks.add(new Fireball(projectilePosition, velocityX, velocityY, m_fireballDamage, 500, newActionDuration));
		}
		// slash attack
		else if(keyboard.keyDownOnce(KeyEvent.VK_K) && 
				m_skills.get(PlayerSkill.Heavy_Strike) > 0 && 
				(GetMana() >= m_heavyStrikeCost))
		{
			if (m_currentWeapon != WeaponType.Bow)
			{
				DecreaseMana(m_heavyStrikeCost);
			
				
				newAction = true;
				currentAction = Action.Thrust;
				newActionDuration = m_heavyStrikeDuration;
				loopNewAction = false;

				if (m_currentWeapon == WeaponType.Dagger)
				{
					newActionDuration /= 2;
					currentAction = Action.Slash;
				}
				Slash slash = new Slash(m_position, m_heavyStrikeDamage, newActionDuration,  m_currentWeapon, currentDirection);
				m_attacks.add(slash);
			}
		}
		// multishot
		else if(keyboard.keyDownOnce(KeyEvent.VK_L) &&
				m_skills.get(PlayerSkill.Multishot) > 0 && 
				 (GetMana() >= m_multiShotCost) &&
				m_currentWeapon == WeaponType.Bow)
		{
			DecreaseMana(m_multiShotCost);
			
			// bow attack
			newAction = true;
			currentAction = Action.Bow;
			newActionDuration = m_multiShotDuration;
			loopNewAction = false;

			Slash arrow = new Slash(m_position, 0, newActionDuration,  WeaponType.Arrow, currentDirection);
			Slash bow = new Slash(m_position, 0, newActionDuration,  WeaponType.Bow, currentDirection);
			
			float velocityX = 0.0f;
			float velocityY = 0.0f;
			
			float xOffset = 0;
			float yOffset = 0;
					
			if (currentDirection == Direction.NORTH)
			{
				velocityY = -1f * m_fireballSpeed;
				
				xOffset = 40;
			}
			else if (currentDirection == Direction.WEST)
			{
				velocityX = -1f * m_fireballSpeed;
				
				yOffset = 40;
			}
			else if (currentDirection == Direction.SOUTH)
			{
				velocityY = m_fireballSpeed;
				
				xOffset = 40;
			}
			else if (currentDirection == Direction.EAST)
			{
				velocityX = m_fireballSpeed;
				
				yOffset = 40;
			}
			
			Vector2D projectilePosition = new Vector2D(m_position.GetX() + m_currentImage.getWidth()/2 - 10, m_position.GetY() + m_currentImage.getHeight()/2);
			
			projectilePosition.SetY(projectilePosition.GetY() - 8);
			m_attacks.add(new Arrow(projectilePosition, velocityX, velocityY, m_multiShotDamage, 1000, newActionDuration, currentDirection));
			
			projectilePosition.AddY(yOffset);
			projectilePosition.AddX(xOffset);
			m_attacks.add(new Arrow(projectilePosition, velocityX, velocityY, m_multiShotDamage, 1000, newActionDuration, currentDirection));
			
			projectilePosition.AddY(yOffset * -2.0f);
			projectilePosition.AddX(xOffset * -2.0f);
			m_attacks.add(new Arrow(projectilePosition, velocityX, velocityY, m_multiShotDamage, 1000, newActionDuration, currentDirection));
			
			m_attacks.add(arrow);
			m_attacks.add(bow);
		}
		else if (keyboard.keyDownOnce(KeyEvent.VK_SPACE))
		{
			if (m_currentWeapon != WeaponType.Bow)
			{
				// slash attack
				newAction = true;
				currentAction = Action.Thrust;
				newActionDuration = GetAttackSpeed();
				loopNewAction = false;
				
				if (m_currentWeapon == WeaponType.Dagger)
				{
					newActionDuration /= 2;
					currentAction = Action.Slash;
				}

				Slash slash = new Slash(m_position, GetDamage(), newActionDuration,  m_currentWeapon, currentDirection);
				m_attacks.add(slash);
			}
			else if (m_currentWeapon == WeaponType.Bow)
			{
				// bow attack
				newAction = true;
				currentAction = Action.Bow;
				newActionDuration = GetAttackSpeed();
				loopNewAction = false;

				Slash arrow = new Slash(m_position, 0, newActionDuration,  WeaponType.Arrow, currentDirection);
				Slash bow = new Slash(m_position, 0, newActionDuration,  WeaponType.Bow, currentDirection);
				
				float velocityX = 0.0f;
				float velocityY = 0.0f;
						
				if (currentDirection == Direction.NORTH)
				{
					velocityY = -1f * m_fireballSpeed;
				}
				else if (currentDirection == Direction.WEST)
				{
					velocityX = -1f * m_fireballSpeed;
				}
				else if (currentDirection == Direction.SOUTH)
				{
					velocityY = m_fireballSpeed;
				}
				else if (currentDirection == Direction.EAST)
				{
					velocityX = m_fireballSpeed;
				}
				
				Vector2D projectilePosition = new Vector2D(m_position.GetX() + m_currentImage.getWidth()/2 - 16, m_position.GetY() + m_currentImage.getHeight()/2 - 8);
				m_attacks.add(new Arrow(projectilePosition, velocityX, velocityY, GetDamage(), 1000, newActionDuration, currentDirection));
				m_attacks.add(arrow);
				m_attacks.add(bow);
			}
		}
		else
		{
			// Character movement. Facing only changes on single button direction or single button + deadlock otherwise maintain same facing.
			// Allows for "strafing" while heading in one direction. 
			// If you can think of a better way to do this or if my commenting can be improved let me know! curlymon.mh@gmail.com 
			
			// A+D deadlock resolution
			if ((keyboard.keyDown(KeyEvent.VK_A) ||
				 keyboard.keyDown(KeyEvent.VK_LEFT) ||
				 keyboard.keyDown(KeyEvent.VK_KP_LEFT)) &&
				(keyboard.keyDown(KeyEvent.VK_D) ||
				 keyboard.keyDown(KeyEvent.VK_RIGHT) ||
				 keyboard.keyDown(KeyEvent.VK_KP_RIGHT)))
			{
				if ((keyboard.keyDown(KeyEvent.VK_W) ||
					 keyboard.keyDown(KeyEvent.VK_UP) ||
					 keyboard.keyDown(KeyEvent.VK_KP_UP)) &&
					(keyboard.keyDown(KeyEvent.VK_S) ||
					 keyboard.keyDown(KeyEvent.VK_DOWN) ||
					 keyboard.keyDown(KeyEvent.VK_KP_DOWN)))
				{
					if (currentAction != Action.Idle)
					{
						currentAction = Action.Idle;
						newAction = true;
					}
				}
				else if ((keyboard.keyDown(KeyEvent.VK_W) ||
						  keyboard.keyDown(KeyEvent.VK_UP) ||
						  keyboard.keyDown(KeyEvent.VK_KP_UP)))
				{
					m_acceleration.AddY(-0.01f);
					if (currentDirection != Direction.NORTH)
					{
						currentDirection = Direction.NORTH;
						newAction = true;
					}
					if (currentAction != Action.Walk)
					{
						currentAction = Action.Walk;
						newAction = true;
					}
				}
				else if ((keyboard.keyDown(KeyEvent.VK_S) ||
						  keyboard.keyDown(KeyEvent.VK_DOWN) ||
						  keyboard.keyDown(KeyEvent.VK_KP_DOWN)))
				{
					m_acceleration.AddY(0.01f);
					if (currentDirection != Direction.SOUTH)
					{
						currentDirection = Direction.SOUTH;
						newAction = true;
					}
					if (currentAction != Action.Walk)
					{
						currentAction = Action.Walk;
						newAction = true;
					}
				}
				else
				{
					if (currentAction != Action.Idle)
					{
						currentAction = Action.Idle;
						newAction = true;
					}
				}
			}
			// W+D deadlock resolution
			else if ((keyboard.keyDown(KeyEvent.VK_W) ||
					  keyboard.keyDown(KeyEvent.VK_UP) ||
					  keyboard.keyDown(KeyEvent.VK_KP_UP)) &&
					 (keyboard.keyDown(KeyEvent.VK_S) ||
					  keyboard.keyDown(KeyEvent.VK_DOWN) ||
					  keyboard.keyDown(KeyEvent.VK_KP_DOWN)))
			{
				if ((keyboard.keyDown(KeyEvent.VK_A) ||
					 keyboard.keyDown(KeyEvent.VK_LEFT) ||
					 keyboard.keyDown(KeyEvent.VK_KP_LEFT)))
				{
					m_acceleration.AddX(-0.01f);
					if (currentDirection != Direction.WEST)
					{
						currentDirection = Direction.WEST;
						newAction = true;
					}
					if (currentAction != Action.Walk)
					{
						currentAction = Action.Walk;
						newAction = true;
					}
				}
				else if ((keyboard.keyDown(KeyEvent.VK_D) ||
						  keyboard.keyDown(KeyEvent.VK_RIGHT) ||
						  keyboard.keyDown(KeyEvent.VK_KP_RIGHT)))
				{
					m_acceleration.AddX(0.01f);
					if (currentDirection != Direction.EAST)
					{
						currentDirection = Direction.EAST;
						newAction = true;
					}
					if (currentAction != Action.Walk)
					{
						currentAction = Action.Walk;
						newAction = true;
					}
				}
				else
				{
					if (currentAction != Action.Idle)
					{
						currentAction = Action.Idle;
						newAction = true;
					}
				}
			}
			
			//  NorthEast resolution
			else if ((keyboard.keyDown(KeyEvent.VK_W) ||
					  keyboard.keyDown(KeyEvent.VK_UP) ||
					  keyboard.keyDown(KeyEvent.VK_KP_UP)) &&
					 (keyboard.keyDown(KeyEvent.VK_D) ||
					  keyboard.keyDown(KeyEvent.VK_RIGHT) ||
					  keyboard.keyDown(KeyEvent.VK_KP_RIGHT)))
			{
				m_acceleration.AddX(0.01f);
				m_acceleration.AddY(-0.01f);
				if (currentAction != Action.Walk)
				{
					currentAction = Action.Walk;
					newAction = true;
				}
			}
			
			// NorthWest resolution
			else if ((keyboard.keyDown(KeyEvent.VK_W) ||
					  keyboard.keyDown(KeyEvent.VK_UP) ||
					  keyboard.keyDown(KeyEvent.VK_KP_UP)) &&
					 (keyboard.keyDown(KeyEvent.VK_A) ||
					  keyboard.keyDown(KeyEvent.VK_LEFT) ||
					  keyboard.keyDown(KeyEvent.VK_KP_LEFT)))
			{
				m_acceleration.AddX(-0.01f);
				m_acceleration.AddY(-0.01f);
				if (currentAction != Action.Walk)
				{
					currentAction = Action.Walk;
					newAction = true;
				}
			}
			
			// SouthEast resolution
			else if ((keyboard.keyDown(KeyEvent.VK_S) ||
					  keyboard.keyDown(KeyEvent.VK_DOWN) ||
					  keyboard.keyDown(KeyEvent.VK_KP_DOWN)) &&
					 (keyboard.keyDown(KeyEvent.VK_D) ||
					  keyboard.keyDown(KeyEvent.VK_RIGHT) ||
					  keyboard.keyDown(KeyEvent.VK_KP_RIGHT)))
			{
				m_acceleration.AddX(0.01f);
				m_acceleration.AddY(0.01f);
				if (currentAction != Action.Walk)
				{
					currentAction = Action.Walk;
					newAction = true;
				}
			}
			
			// SouthWest resolution
			else if ((keyboard.keyDown(KeyEvent.VK_S) ||
					  keyboard.keyDown(KeyEvent.VK_DOWN) ||
					  keyboard.keyDown(KeyEvent.VK_KP_DOWN)) &&
					 (keyboard.keyDown(KeyEvent.VK_A) ||
					  keyboard.keyDown(KeyEvent.VK_LEFT) ||
					  keyboard.keyDown(KeyEvent.VK_KP_LEFT)))
			{
				m_acceleration.AddX(-0.01f);
				m_acceleration.AddY(0.01f);
				if (currentAction != Action.Walk)
				{
					currentAction = Action.Walk;
					newAction = true;
				}
			}
			
			// North resolution
			else if ((keyboard.keyDown(KeyEvent.VK_W) ||
					  keyboard.keyDown(KeyEvent.VK_UP) ||
					  keyboard.keyDown(KeyEvent.VK_KP_UP)))
			{
				m_acceleration.AddY(-0.01f);
				if (currentDirection != Direction.NORTH)
				{
					currentDirection = Direction.NORTH;
					newAction = true;
				}
				if (currentAction != Action.Walk)
				{
					currentAction = Action.Walk;
					newAction = true;
				}
			}
			
			// South resolution
			else if ((keyboard.keyDown(KeyEvent.VK_S) ||
					  keyboard.keyDown(KeyEvent.VK_DOWN) ||
					  keyboard.keyDown(KeyEvent.VK_KP_DOWN)))
			{
				m_acceleration.AddY(0.01f);
				if (currentDirection != Direction.SOUTH)
				{
					currentDirection = Direction.SOUTH;
					newAction = true;
				}
				if (currentAction != Action.Walk)
				{
					currentAction = Action.Walk;
					newAction = true;
				}
			}
			
			// East resolution
			else if ((keyboard.keyDown(KeyEvent.VK_D) ||
					  keyboard.keyDown(KeyEvent.VK_RIGHT) ||
					  keyboard.keyDown(KeyEvent.VK_KP_RIGHT)))
			{
				m_acceleration.AddX(0.01f);
				if (currentDirection != Direction.EAST)
				{
					currentDirection = Direction.EAST;
					newAction = true;
				}
				if (currentAction != Action.Walk)
				{
					currentAction = Action.Walk;
					newAction = true;
				}
			}
			
			// West resolution
			else if ((keyboard.keyDown(KeyEvent.VK_A) ||
					  keyboard.keyDown(KeyEvent.VK_LEFT) ||
					  keyboard.keyDown(KeyEvent.VK_KP_LEFT)))
			{
				m_acceleration.AddX(-0.01f);
				if (currentDirection != Direction.WEST)
				{
					currentDirection = Direction.WEST;
					newAction = true;
				}
				if (currentAction != Action.Walk)
				{
					currentAction = Action.Walk;
					newAction = true;
				}
			}
		}
				
		if (newAction)
		{
			m_animator.BeginNewAction(currentDirection, currentAction, newActionDuration, loopNewAction);
			
		}
		else if (currentAction == Action.Walk
				&& (m_velocity.GetX() < 0.001f && m_velocity.GetX() > (-0.001f))
				&& (m_velocity.GetY() < 0.001f && m_velocity.GetY() > (-0.001f)))
		{
			m_animator.BeginNewAction(currentDirection, Action.Idle, 100, true);
		}
	}

	private void LoadNewTextures(BodyPart part, ItemType type)
	{
		String fileName = GetFileName(part, type);
		
		// Set animation frame data
		int frameHeight = 64;
		int frameWidth = 64;
				
		// Order directions for looping
		LinkedList<Direction> directionsInOrder = new LinkedList<Direction>();
		directionsInOrder.add(Direction.NORTH);
		directionsInOrder.add(Direction.WEST);
		directionsInOrder.add(Direction.SOUTH);
		directionsInOrder.add(Direction.EAST);
				
		LoadAnimations(directionsInOrder, frameHeight, frameWidth, fileName, part);
	}
	
	private String GetFileName(BodyPart part, ItemType type)
	{
		String fileName = "";
		
		if (type == ItemType.ARMOR_CLOTH)
		{
			if (part == BodyPart.TORSO)
			{
				fileName = "TORSO_robe_shirt_brown";
			}
			else if( part == BodyPart.BELT )
			{
				fileName = "BELT_rope";
			}
			else if( part == BodyPart.FEET )
			{
				// Yay, bare feet!
			}
			else if( part == BodyPart.HANDS )
			{
				// Yay, bare hands!
			}
			else if( part == BodyPart.HEAD )
			{
				fileName = "HEAD_robe_hood";
			}
			else if( part == BodyPart.LEGS )
			{
				fileName = "LEGS_robe_skirt";
			}
		}
		else if (type == ItemType.ARMOR_LEATHER)
		{
			if (part == BodyPart.TORSO)
			{
				//TORSO_leather_armor_bracers.png
				//TORSO_leather_armor_shirt_white.png
				//TORSO_leather_armor_shoulders.png
				fileName = "TORSO_leather_armor_torso";
			}
			else if( part == BodyPart.BELT )
			{
				fileName = "BELT_leather";
			}
			else if( part == BodyPart.FEET )
			{
				fileName = "FEET_shoes_brown";
			}
			else if( part == BodyPart.HANDS )
			{
				// Still bare hands
			}
			else if( part == BodyPart.HEAD )
			{
				fileName = "HEAD_leather_armor_hat";
			}
			else if( part == BodyPart.LEGS )
			{
				fileName = "LEGS_pants_greenish";
			}
		}
		else if (type == ItemType.ARMOR_CHAIN)
		{
			if (part == BodyPart.TORSO)
			{
				//TORSO_chain_armor_jacket_purple.png
				fileName = "TORSO_chain_armor_torso";
			}
			else if( part == BodyPart.BELT )
			{
				fileName = "BELT_leather";
			}
			else if( part == BodyPart.FEET )
			{
				fileName = "FEET_shoes_brown";
			}
			else if( part == BodyPart.HANDS )
			{
			}
			else if( part == BodyPart.HEAD )
			{
				fileName = "HEAD_chain_armor_helmet";
				//HEAD_chain_armor_hood.png
			}
			else if( part == BodyPart.LEGS )
			{
				fileName = "LEGS_pants_greenish";
			}
		}
		else if (type == ItemType.ARMOR_PLATE)
		{
			if (part == BodyPart.TORSO)
			{
				//TORSO_plate_armor_arms_shoulders.png
				fileName = "TORSO_plate_armor_torso";
			}
			else if( part == BodyPart.BELT )
			{
				fileName = "BELT_leather";
			}
			else if( part == BodyPart.FEET )
			{
				fileName = "FEET_plate_armor_shoes";
			}
			else if( part == BodyPart.HANDS )
			{
				fileName = "HANDS_plate_armor_gloves";
			}
			else if( part == BodyPart.HEAD )
			{
				fileName = "HEAD_plate_armor_helmet";
			}
			else if( part == BodyPart.LEGS )
			{
				fileName = "LEGS_plate_armor_pants";
			}
		}
		
		fileName += ".png";

		return fileName;
	}
	
	/**
	 * Load all animations for a sprite sheet
	 * @param directionsInOrder - the ordered facing directions of the frames of animation
	 * @param frameHeight - the height of each frame, in pixels
	 * @param frameWidth - the width of each frame, in pixels
	 * @param pngFilename - the name of the file (TORSO_robe_shirt_brown.png, for example)
	 * @param part - the body part for which this texture applies to
	 */
	private void LoadAnimations(Iterable<Direction> directionsInOrder, int frameHeight, int frameWidth, String pngFilename, BodyPart part)
	{
		// Walking
		m_animator.LoadSpriteSheet(".\\Resources\\Images\\walkcycle\\" + pngFilename);
		m_animator.AddAnimations(part, Action.Walk, directionsInOrder, frameHeight, frameWidth, 9);

		// Idle
		m_animator.AddAnimations(part, Action.Idle, directionsInOrder, frameHeight, frameWidth, 1);
						
		// Casting
		m_animator.LoadSpriteSheet(".\\Resources\\Images\\spellcast\\" + pngFilename);		
		m_animator.AddAnimations(part, Action.Spell, directionsInOrder, frameHeight, frameWidth, 7);
				
		// Slashing
		m_animator.LoadSpriteSheet(".\\Resources\\Images\\slash\\" + pngFilename);		
		m_animator.AddAnimations(part, Action.Slash, directionsInOrder, frameHeight, frameWidth, 6);

		// Thrusting
		m_animator.LoadSpriteSheet(".\\Resources\\Images\\thrust\\" + pngFilename);		
		m_animator.AddAnimations(part, Action.Thrust, directionsInOrder, frameHeight, frameWidth, 8);
		
		// Shoot Bow
		m_animator.LoadSpriteSheet(".\\Resources\\Images\\bow\\" + pngFilename);		
		m_animator.AddAnimations(part, Action.Bow, directionsInOrder, frameHeight, frameWidth, 13);
		
		// Death
		m_animator.LoadSpriteSheet(".\\Resources\\Images\\hurt\\" + pngFilename);
		m_animator.AddAnimation(part, Action.Fall, Direction.SOUTH, 0, 0, frameHeight, frameWidth, 6);
	}
}
