/**Copyright  2012 held by
 * Robert Williams
 * Robert Poland
 * Marc Hayes
 * Coen Smits
 * Justin Becker
 * All rights reserved.
 * 
 * 
 ** This file is part of Bloodmoon.

    Bloodmoon is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bloodmoon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bloodmoon.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.event.KeyEvent;
import java.util.HashMap;


public class PlayerSkillOverlay extends ScrollOverlay 
{
	// Members
	private PlayerCharacter m_player = null;
	private int m_selectedIndex;
	private PlayerSkill m_selectedSkill;
		
	public PlayerSkillOverlay(GameEngine gameEngine, PlayerCharacter player) {
		super(gameEngine);
		
		m_player = player;
		
		m_selectedIndex = 0;
	}

	@Override
	public void Paint(Graphics2D graphics)
	{
		super.Paint(graphics);
		
		int y = m_y + m_scrollBarTopHeight;
		
		y += 20;
		graphics.drawString("SKILLS", m_x + (m_scrollBarWidth / 2) - 60, y);

		y += 20;
		
		int index = 0;
		
		HashMap<PlayerSkill, Integer> playerSkills = m_player.GetPlayerSkills();
		
		// Do your custom inventory rendering in here
		for( PlayerSkill skill : PlayerSkill.values())
		{
			y += 20;

			graphics.setColor(Color.BLACK);
						
			graphics.drawString(skill.toString() + " - Level " + playerSkills.get(skill), m_x + 40, y);
			
			if (index == m_selectedIndex)
			{
				// Highlight this one
				graphics.setColor(Color.BLUE);
				graphics.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.25f));
				graphics.fillRect(m_x+10, y-13, 200, 15);
			
				graphics.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 1.0f));
				graphics.drawRect(m_x+10, y-13, 200, 15);
				
				m_selectedSkill = skill;
			}
			
			index++;
		}
		
		y += 40;
		
		graphics.setColor(Color.BLACK);
		graphics.drawString("Remaining skill points: " + m_player.GetSkillPoints(), m_x + 40, y);
	}
	
	@Override
	public void Update(long elapsedGameTime, KeyboardInput keyboard)
	{
		super.Update(elapsedGameTime, keyboard);
		
		if( keyboard.keyDownOnce(KeyEvent.VK_U))
		{
			m_gameEngine.setGameMode();
		}
		else if (keyboard.keyDownOnce(KeyEvent.VK_S) ||
				keyboard.keyDownOnce(KeyEvent.VK_KP_DOWN) ||
				keyboard.keyDownOnce(KeyEvent.VK_DOWN))
		{			
			// Change the 5 here to [max number of skills-1]
			// Move menu button down			
			if( m_selectedIndex < 5 )
			{
				m_selectedIndex++;
			}
		}
		else if (keyboard.keyDownOnce(KeyEvent.VK_W) ||
					 keyboard.keyDownOnce(KeyEvent.VK_KP_UP) ||
					 keyboard.keyDownOnce(KeyEvent.VK_UP))
		{
			// Move menu button up
			if( m_selectedIndex > 0 )
			{
				m_selectedIndex--;
			}
		}
		else if (keyboard.keyDownOnce(KeyEvent.VK_ENTER))
		{
			if(m_selectedSkill == null)
			{
				return;
			}
			
			m_player.LearnSkill(m_selectedSkill);
		}
	}
}
