/**Copyright  2012 held by
 * Robert Williams
 * Robert Poland
 * Marc Hayes
 * Coen Smits
 * Justin Becker
 * All rights reserved.
 * 
 * 
 * This file is part of Bloodmoon.

    Bloodmoon is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bloodmoon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bloodmoon.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.Graphics2D;



/**
 * Represents any ranged attack, such as arrows or fireballs
 * @author rpoland
 */
public abstract class Projectile extends Attack
{	
	long m_delay;
	
	public Projectile(Vector2D initialPosition, float accelerationX, float accelerationY, int damage, long lifeSpan, long delay)
	{
		// Invoke base (super) class constructor
		super(initialPosition, accelerationX, accelerationY, damage, lifeSpan);
		
		m_delay = delay;
	}
	
	@Override
	public void Update(long elapsedGameTime)
	{
		if (m_delay > 0L)
		{
			m_delay -= elapsedGameTime;
		}
		else
		{
			super.Update(elapsedGameTime);
			super.UpdatePosition(elapsedGameTime);
		}
	}
	
	@Override
	protected void UpdatePosition(long elapsedGameTime)
	{
		if (m_delay <= 0L)
		{
			super.UpdatePosition(elapsedGameTime);
		}
	}
	
	public void Paint(Graphics2D g)
	{
		if (m_delay > 0L)
		{
			return;			
		}
		
		super.Paint(g);
	}
}
