/**Copyright  2012 held by
 * Robert Williams
 * Robert Poland
 * Marc Hayes
 * Coen Smits
 * Justin Becker
 * All rights reserved.
 * 
 * 
 * This file is part of Bloodmoon.

    Bloodmoon is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bloodmoon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bloodmoon.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.Rectangle;

public class Slash extends Attack
	{		
		// ==============================================
		//                  Members
		// ==============================================
		private WeaponType m_weaponType;
		private Direction m_direction;
		private Action m_action = Action.Idle;

		// ==============================================
		//                Constructors
		// ==============================================		
		public Slash(Vector2D initialPosition, int damage, long lifeSpan, WeaponType weaponType, Direction dir)
		{
			super(initialPosition, 0.0f, 0.0f, damage, lifeSpan);
			
			m_weaponType = weaponType;
			m_direction = dir;
			
			InitializeSprites();
			InitializeBoundingBox();
			
			SetDebugInfo("Damage is: " + damage);
			
//			SetDebugColor(Color.RED);
//			SetShowDebug(true);
		}

		// ==============================================
		//                 Overrides
		// ==============================================
		@Override
		protected void InitializeSprites()
		{
			if (WeaponType.Dagger == m_weaponType)
			{
				LoadSwordSlashAnimation();
			}
			else if (WeaponType.Spear == m_weaponType)
			{
				LoadSpearAnimation();
			}
			else if (WeaponType.Staff == m_weaponType)
			{
				LoadStaffAnimation();
			}
			else if (WeaponType.Bow == m_weaponType)
			{
				LoadBowAnimation();
			}
			else if (WeaponType.Arrow == m_weaponType)
			{
				LoadArrowAnimation();
			}
			
			m_currentImage = m_animator.CalculateNextFrameLinear(0);
		}

		
		@Override
		protected void InitializeStartingVectors() 
		{	
		}
	
		@Override
		protected void Initialize()
		{			
		}
		
		@Override
		protected void InitializeBoundingBox()
		{
			if (m_position == null || m_currentImage == null)
			{
				return;
			}
						
			int imageWidth = m_currentImage.getWidth();
			int imageHeight = m_currentImage.getHeight();

			// Reduce the size of the attack by 50%
			int boundingBoxWidth = imageWidth / 2;
			int boundingBoxHeight = imageHeight / 2;
			
			int posX = (int)m_position.GetX();
			int posY = (int)m_position.GetY();
			
			// Adjust the location of the attack, according to direction
			if (Direction.NORTH == m_direction)
			{
				posX += boundingBoxWidth / 2;
				posY -= boundingBoxHeight / 2;
			}
			else if (Direction.SOUTH == m_direction)
			{
				posX += boundingBoxWidth / 2;
				posY += boundingBoxHeight * 2;
			}
			else if (Direction.EAST == m_direction)
			{
				posX += boundingBoxWidth * 1.5;
				posY += boundingBoxHeight * 0.75;
			}
			else if (Direction.WEST == m_direction)
			{
				posX -= boundingBoxWidth / 2;
				posY += boundingBoxHeight * 0.75;
			}
			
			m_boundingBox = new Rectangle(posX, posY, boundingBoxWidth, boundingBoxHeight);
			
			m_boundingBoxOffSetX = 0;
			m_boundingBoxOffSetY = 0;
			
			SetDebugInfo("m_boundingBox is " + m_boundingBox.width + " x " + m_boundingBox.height);
		}
		
		// ==============================================
		//                  Methods
		// ==============================================
				
		protected Action GetAction()
		{
			return m_action;
		}
		
		private void LoadSpearAnimation()
		{
			LoadAnimation(".\\Resources\\Images\\thrust\\WEAPON_spear.png", 8);
		}
		
		private void LoadStaffAnimation()
		{
			LoadAnimation(".\\Resources\\Images\\thrust\\WEAPON_staff.png", 8);
		}
		
		private void LoadSwordSlashAnimation()
		{
			LoadAnimation(".\\Resources\\Images\\slash\\WEAPON_dagger.png", 6);
		}
		
		private void LoadBowAnimation()
		{
			LoadAnimation(".\\Resources\\Images\\bow\\WEAPON_bow.png", 13);
		}
		
		private void LoadArrowAnimation()
		{
			LoadAnimation(".\\Resources\\Images\\bow\\WEAPON_arrow.png", 13);
		}
		
		private void LoadAnimation(String filepath, int frameCount)
		{
			m_animator.LoadSpriteSheet(filepath);
			
			int startX = 0, startY = 0;
			
			if (m_direction == Direction.WEST)
			{
				startY = 64;
			}
			else if (m_direction == Direction.SOUTH)
			{
				startY = 64*2;
			}
			else if (m_direction == Direction.EAST)
			{
				startY = 64*3;
			}
			m_animator.AddAnimation(Action.Idle, m_direction, startX, startY, 64, 64, frameCount);
			m_animator.Finish();
			
			m_animator.BeginNewAction(m_direction, Action.Idle, m_lifeSpan, false);
		}
	}